"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_utils_1 = require("@pnpm/cli-utils");
const outdated_1 = require("@pnpm/outdated");
const table_1 = require("@zkochan/table");
const chalk_1 = __importDefault(require("chalk"));
const isEmpty_js_1 = __importDefault(require("ramda/src/isEmpty.js"));
const sortWith_js_1 = __importDefault(require("ramda/src/sortWith.js"));
const outdated_2 = require("./outdated");
const utils_1 = require("./utils");
const DEP_PRIORITY = {
    dependencies: 1,
    devDependencies: 2,
    optionalDependencies: 0,
};
const COMPARATORS = [
    ...utils_1.DEFAULT_COMPARATORS,
    (o1, o2) => DEP_PRIORITY[o1.belongsTo] - DEP_PRIORITY[o2.belongsTo],
];
exports.default = async (pkgs, params, opts) => {
    const outdatedMap = {};
    const outdatedPackagesByProject = await (0, outdated_1.outdatedDepsOfProjects)(pkgs, params, {
        ...opts,
        fullMetadata: opts.long,
        retry: {
            factor: opts.fetchRetryFactor,
            maxTimeout: opts.fetchRetryMaxtimeout,
            minTimeout: opts.fetchRetryMintimeout,
            retries: opts.fetchRetries,
        },
        timeout: opts.fetchTimeout,
    });
    for (let i = 0; i < outdatedPackagesByProject.length; i++) {
        const { dir, manifest } = pkgs[i];
        outdatedPackagesByProject[i].forEach((outdatedPkg) => {
            const key = JSON.stringify([outdatedPkg.packageName, outdatedPkg.current, outdatedPkg.belongsTo]);
            if (!outdatedMap[key]) {
                outdatedMap[key] = { ...outdatedPkg, dependentPkgs: [] };
            }
            outdatedMap[key].dependentPkgs.push({ location: dir, manifest });
        });
    }
    if ((0, isEmpty_js_1.default)(outdatedMap))
        return { output: '', exitCode: 0 };
    if (opts.table !== false) {
        return { output: renderOutdatedTable(outdatedMap, opts), exitCode: 1 };
    }
    return { output: renderOutdatedList(outdatedMap, opts), exitCode: 1 };
};
function renderOutdatedTable(outdatedMap, opts) {
    const columnNames = [
        'Package',
        'Current',
        'Latest',
        'Dependents',
    ];
    const columnFns = [
        outdated_2.renderPackageName,
        outdated_2.renderCurrent,
        outdated_2.renderLatest,
        dependentPackages,
    ];
    if (opts.long) {
        columnNames.push('Details');
        columnFns.push(outdated_2.renderDetails);
    }
    // Avoid the overhead of allocating a new array caused by calling `array.map()`
    for (let i = 0; i < columnNames.length; i++)
        columnNames[i] = chalk_1.default.blueBright(columnNames[i]);
    const data = [
        columnNames,
        ...sortOutdatedPackages(Object.values(outdatedMap))
            .map((outdatedPkg) => columnFns.map((fn) => fn(outdatedPkg))),
    ];
    return (0, table_1.table)(data, {
        ...cli_utils_1.TABLE_OPTIONS,
        columns: {
            ...cli_utils_1.TABLE_OPTIONS.columns,
            // Dependents column:
            3: {
                width: (0, outdated_2.getCellWidth)(data, 3, 30),
                wrapWord: true,
            },
        },
    });
}
function renderOutdatedList(outdatedMap, opts) {
    return sortOutdatedPackages(Object.values(outdatedMap))
        .map((outdatedPkg) => {
        let info = `${chalk_1.default.bold((0, outdated_2.renderPackageName)(outdatedPkg))}
${(0, outdated_2.renderCurrent)(outdatedPkg)} ${chalk_1.default.grey('=>')} ${(0, outdated_2.renderLatest)(outdatedPkg)}`;
        const dependents = dependentPackages(outdatedPkg);
        if (dependents) {
            info += `\n${chalk_1.default.bold(outdatedPkg.dependentPkgs.length > 1
                ? 'Dependents:'
                : 'Dependent:')} ${dependents}`;
        }
        if (opts.long) {
            const details = (0, outdated_2.renderDetails)(outdatedPkg);
            if (details) {
                info += `\n${details}`;
            }
        }
        return info;
    })
        .join('\n\n') + '\n';
}
function dependentPackages({ dependentPkgs }) {
    return dependentPkgs
        .map(({ manifest, location }) => manifest.name ?? location)
        .sort()
        .join(', ');
}
function sortOutdatedPackages(outdatedPackages) {
    return (0, sortWith_js_1.default)(COMPARATORS, outdatedPackages.map(outdated_2.toOutdatedWithVersionDiff));
}
//# sourceMappingURL=recursive.js.map